/*  Copyright (C) 2015 Michele Santullo
 *
 *  This file is part of DuckScraper.
 *
 *  DuckScraper is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DuckScraper is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with DuckScraper.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "html_pool.hpp"
#include "htmlretrieve.hpp"
#include "read_all.hpp"
#include <string>
#include <memory>
#include <utility>
#include <iostream>

namespace duck {
	HtmlPool::HtmlPool (std::string&& agent_name, std::string&& src_charset) :
		m_agent(std::move(agent_name)),
		m_src_charset(std::move(src_charset)),
		m_iconv(m_src_charset, "utf-8", IconvWrapper::ModeIgnore)
	{
	}

	HtmlPool::~HtmlPool() noexcept = default;

	auto HtmlPool::OnResourceLoad (ResourceObjectParameterType parRes) -> ResourceType* {
		std::cout << "OnResourceLoad(): fetching html from \"" << parRes << "\"\n";

		std::unique_ptr<std::string> utf8_html;
		if (parRes == "-") {
			utf8_html = std::make_unique<std::string>(read_all(std::cin));
		}
		else {
			utf8_html = std::make_unique<std::string>(
				fetch_html(parRes, m_agent, false, false)
			);
		}

		*utf8_html = duck::clean_html(m_iconv.conv_char(*utf8_html), "utf-8");
		return utf8_html.release();
	}

	void HtmlPool::OnResourceDestroy (ResourceNameParamType, ResourceType* parRes) noexcept {
		delete parRes;
	}

	auto HtmlPool::GetResourceNameFromResourceObject (ResourceObjectParameterType parRes) -> ResourceNameType {
		return parRes;
	}
} //namespace duck
