/*  Copyright (C) 2015 Michele Santullo
 *
 *  This file is part of DuckScraper.
 *
 *  DuckScraper is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DuckScraper is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with DuckScraper.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef id8D3B62D447574A23A82F8E9C60A629BD
#define id8D3B62D447574A23A82F8E9C60A629BD

#include <cstddef>

namespace duckutil {
	template <typename T, std::size_t IndexBitSize, std::size_t SaltBitSize=sizeof(T)*8-IndexBitSize>
	class SaltedIndex {
		static_assert((SaltBitSize + IndexBitSize) == sizeof(T) * 8, "Type size is too small");
		static_assert(SaltBitSize > 0, "Invalid salt size");
		static_assert(IndexBitSize > 0, "Invalid index size");
	public:
		enum {
			SaltSize = SaltBitSize,
			IndexSize = IndexBitSize,
			MaxSalt = (1 << SaltBitSize) - 1,
			MaxIndex = (1 << IndexBitSize) - 1
		};

		SaltedIndex ( void );
		SaltedIndex ( const SaltedIndex& parOther );
		explicit SaltedIndex ( T parIndex );
		SaltedIndex ( T parSalt, T parIndex );
		~SaltedIndex ( void );

		T GetSaltOnly ( void ) const { return m_salt; }
		T GetIndexOnly ( void ) const { return m_index; }
		T GetSaltedIndex ( void ) const { return m_saltedIndex; }
		void SetSalt ( T parSalt );
		void SetIndex ( T parIndex );
		T IncreaseSalt ( void );

		bool operator== ( const SaltedIndex& parOther ) const { return GetSaltedIndex() == parOther.GetSaltedIndex(); }
		bool operator!= ( const SaltedIndex& parOther ) const { return GetSaltedIndex() != parOther.GetSaltedIndex(); }
		bool operator< ( const SaltedIndex& parOther ) const { return GetSaltedIndex() < parOther.GetSaltedIndex(); }

	private:
		union {
			struct {
				T m_index : IndexBitSize;
				T m_salt : SaltBitSize;
			};
			T m_saltedIndex;
		};
	};
} //namespace duckutil

#include "SaltedIndex.inl"

#endif
