/*  Copyright (C) 2015 Michele Santullo
 *
 *  This file is part of DuckScraper.
 *
 *  DuckScraper is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DuckScraper is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with DuckScraper.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace duckutil {
	///-------------------------------------------------------------------------
	///-------------------------------------------------------------------------
	template <typename T, std::size_t IndexBitSize, std::size_t SaltBitSize>
	SaltedIndex<T, IndexBitSize, SaltBitSize>::SaltedIndex() :
		m_saltedIndex(0)
	{
	}

	///-------------------------------------------------------------------------
	///-------------------------------------------------------------------------
	template <typename T, std::size_t IndexBitSize, std::size_t SaltBitSize>
	SaltedIndex<T, IndexBitSize, SaltBitSize>::SaltedIndex (const SaltedIndex& parOther) :
		m_saltedIndex(parOther.GetSaltedIndex())
	{
	}

	///-------------------------------------------------------------------------
	///-------------------------------------------------------------------------
	template <typename T, std::size_t IndexBitSize, std::size_t SaltBitSize>
	SaltedIndex<T, IndexBitSize, SaltBitSize>::SaltedIndex (T parIndex) :
		m_saltedIndex(parIndex)
	{
		Assert(m_saltedIndex <= MaxIndex);
		Assert(m_salt == 0);
	}

	///-------------------------------------------------------------------------
	///-------------------------------------------------------------------------
	template <typename T, std::size_t IndexBitSize, std::size_t SaltBitSize>
	SaltedIndex<T, IndexBitSize, SaltBitSize>::SaltedIndex (T parSalt, T parIndex) :
		m_index(parIndex),
		m_salt(parSalt)
	{
		Assert(parSalt <= MaxSalt);
		Assert(parIndex <= MaxIndex);
	}

	///-------------------------------------------------------------------------
	///-------------------------------------------------------------------------
	template <typename T, std::size_t IndexBitSize, std::size_t SaltBitSize>
	SaltedIndex<T, IndexBitSize, SaltBitSize>::~SaltedIndex() {
	}

	///-------------------------------------------------------------------------
	///-------------------------------------------------------------------------
	template <typename T, std::size_t IndexBitSize, std::size_t SaltBitSize>
	void SaltedIndex<T, IndexBitSize, SaltBitSize>::SetSalt (T parSalt) {
		Assert(parSalt <= MaxSalt);
		m_salt = parSalt;
	}

	///-------------------------------------------------------------------------
	///-------------------------------------------------------------------------
	template <typename T, std::size_t IndexBitSize, std::size_t SaltBitSize>
	void SaltedIndex<T, IndexBitSize, SaltBitSize>::SetIndex (T parIndex) {
		Assert(parIndex <= MaxIndex);
		m_index = parIndex;
	}

	///-------------------------------------------------------------------------
	///-------------------------------------------------------------------------
	template <typename T, std::size_t IndexBitSize, std::size_t SaltBitSize>
	T SaltedIndex<T, IndexBitSize, SaltBitSize>::IncreaseSalt() {
		Assert(m_salt < MaxSalt);
		++m_salt;
	}
} //namespace duckutil
