/*  Copyright (C) 2017 Michele Santullo
 *
 *  This file is part of DuckScraper.
 *
 *  DuckScraper is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DuckScraper is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with DuckScraper.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "parse.hpp"
#include "scraplang/parse_exports.hpp"
#include "scraplang/scrapgrammar.hpp"
#include "scraplang/element_def.hpp"
#include <boost/spirit/include/qi.hpp>
#include <boost/spirit/include/phoenix_stl.hpp>
#include <boost/spirit/include/phoenix_fusion.hpp>
#include <boost/fusion/adapted/struct.hpp>
#include <boost/fusion/adapted/std_pair.hpp>
#include <utility>
#if !defined(NDEBUG)
#	include <iostream>
#endif
#include <stdexcept>

namespace sp = boost::spirit;

BOOST_FUSION_ADAPT_STRUCT(
	duck::sl::SourceInfo,
	(std::string, value)
	(duck::sl::SourceInfo::Type, type)
)
BOOST_FUSION_ADAPT_STRUCT(
	duck::sl::FromBlock,
	(duck::sl::SourceInfo, source)
	(std::vector<duck::sl::StructItem>, xpaths)
)
BOOST_FUSION_ADAPT_STRUCT(
	duck::sl::StructBlock,
	(std::string, name)
	(std::vector<duck::sl::StructItem>, xpaths)
)
BOOST_FUSION_ADAPT_STRUCT(
	duck::sl::ApplyBlock,
	(std::string, mustache_model)
	(duck::sl::SourceInfo, source)
	(std::vector<duck::sl::StructItem>, xpaths)
)
BOOST_FUSION_ADAPT_STRUCT(
	duck::sl::MustacheBlock,
	(std::string, name)
	(std::string, content)
)
BOOST_FUSION_ADAPT_STRUCT(
	duck::sl::XPathElement,
	(std::string, name)
	(std::optional<std::string>, def_val)
	(std::string, xpath)
)

namespace duck { namespace sl {
	typedef kamokan::IniCommentSkipper<std::string_view::const_iterator> skipper_type;

	std::vector<ScrapNode> parse (std::string_view parData) {
		ScrapGrammar<std::string_view::const_iterator, skipper_type> gramm;
		auto it_start = parData.cbegin();

		std::vector<ScrapNode> retval;
		const bool ok = qi::phrase_parse(it_start, parData.cend(), gramm, skipper_type(), retval);

		std::cout << "parse ok: " << std::boolalpha << ok << '\n';
		std::cout << "end == it: " << std::boolalpha << (parData.cend() == it_start) << '\n';
		std::cout << "begin == it: " << std::boolalpha << (parData.cbegin() == it_start) << '\n';
		std::cout << "parse distance: " << std::distance(parData.cbegin(), it_start) << '\n';
		std::cout << "all distance: " << std::distance(parData.cbegin(), parData.cend()) << " (size: " << parData.size() << ")\n";

		if (parData.cend() != it_start or not ok) {
			throw std::runtime_error("Error parsing input script");
		}
		return retval;
	}
}} //namespace duck::sl
