/*  Copyright (C) 2015 Michele Santullo
 *
 *  This file is part of DuckScraper.
 *
 *  DuckScraper is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DuckScraper is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with DuckScraper.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef id9919CCB09DDD429C8128632F13D370ED
#define id9919CCB09DDD429C8128632F13D370ED

//#include "element_def.hpp"
#include <boost/spirit/include/support_extended_variant.hpp>
#include <string>
#include <vector>
#include <map>
#include <optional>
#include <utility>

namespace duck { namespace sl {
	struct XPathElement {
		std::string name;
		std::optional<std::string> def_val;
		std::string xpath;
	};

	struct StructBlock;

	struct StructItem : boost::spirit::extended_variant<
		XPathElement,
		boost::recursive_wrapper<StructBlock>
	> {
		StructItem() : base_type() {}
		StructItem (const XPathElement& value) : base_type(value) {}
		StructItem (const StructBlock& value) : base_type(value) {}
		using base_type::operator=;
	};

	struct StructBlock {
		std::string name;
		std::vector<StructItem> xpaths;
	};

	struct SourceInfo {
		enum Type { URL, Token };

		std::string value;
		Type type;
	};

	struct FromBlock {
		SourceInfo source;
		std::vector<StructItem> xpaths;
	};

	struct ApplyBlock {
		std::string mustache_model;
		SourceInfo source;
		std::vector<StructItem> xpaths;
	};

	struct MustacheBlock {
		std::string name;
		std::string content;
	};

	struct ScrapNode : boost::spirit::extended_variant<
		boost::recursive_wrapper<std::vector<ScrapNode>>,
		FromBlock,
		ApplyBlock,
		MustacheBlock
	> {
		ScrapNode() : base_type() {}
		ScrapNode (const std::vector<ScrapNode>& value) : base_type(value) {}
		ScrapNode (const FromBlock& value) : base_type(value) {}
		ScrapNode (const ApplyBlock& value) : base_type(value) {}
		ScrapNode (const MustacheBlock& value) : base_type(value) {}
		using base_type::operator=;
	};
}} //namespace duck::sl

#endif
