/*  Copyright (C) 2015 Michele Santullo
 *
 *  This file is part of DuckScraper.
 *
 *  DuckScraper is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DuckScraper is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with DuckScraper.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "xpath_runner.hpp"
#include "xpath.hpp"
#include <cassert>
#include <iostream>

//#define HTML_ALWAYS_STDIN

#if !defined(NDEBUG) && defined(HTML_ALWAYS_STDIN)
#	define HTML_ALWAYS_STDIN_ENABLED
#endif

namespace duck { namespace sl {
	struct XPathRunner::XPathKey {
		XPathKey (const std::string_view& parSrc, const std::string_view& parQuery) :
			source_address(std::string(parSrc)),
			xpath_query(std::string(parQuery))
		{
			assert(not source_address.empty());
		}

		std::string source_address;
		std::string xpath_query;

		bool operator< (const XPathKey& parOther) const {
			return (
				xpath_query == parOther.xpath_query and
				source_address < parOther.source_address
				) or (xpath_query < parOther.xpath_query);
		}
	};

	XPathRunner::XPathRunner (HtmlPoolBaseSP html_pool, XPathPtr& parXPath, std::string&& parDefNamespace) :
		m_cached_results(),
		m_def_namespace(std::move(parDefNamespace)),
		m_pool(html_pool),
		m_xpath(parXPath)
	{
	}

	XPathRunner::~XPathRunner() = default;

	const std::vector<std::string>& XPathRunner::query (
		std::string_view parSrc,
		std::string_view parQuery
	) {
		std::cout << "XPathRunner::query(\"" << parSrc << "\", \"" << parQuery << "\")\"\n";
		auto ins_retval = m_cached_results.insert(std::make_pair(XPathKey(parSrc, parQuery), std::vector<std::string>()));
		const bool inserted = ins_retval.second;
		assert(ins_retval.first != m_cached_results.end());
		std::vector<std::string>& curr_vec = ins_retval.first->second;

		if (inserted) {
#if defined(HTML_ALWAYS_STDIN_ENABLED)
			const auto id = m_pool->GetOrAdd("-");
#else
			const auto id = m_pool->GetOrAdd(parSrc);
#endif
			const std::string* html = m_pool->GetByID(id);

			curr_vec = m_xpath->run_query(*html, std::string(parQuery), m_def_namespace);
			std::cout << "First time for this query, result cached now\n";
		}

		std::cout << "returning " << curr_vec.size() << " items: ";
		for (auto& i : curr_vec) {
			std::cout << '"' << i << "\", ";
		}
		std::cout << '\n';
		return curr_vec;
	}
}} //namespace duck::sl
